import { Box } from 'native-base';
import React, { useRef } from 'react';
import { StyleSheet, Dimensions } from 'react-native';
import { PanGestureHandler, PinchGestureHandler } from 'react-native-gesture-handler';
import Animated, {
    useAnimatedGestureHandler,
    useAnimatedStyle,
    useSharedValue,
} from 'react-native-reanimated';
import { PlanImage, Room } from '../../types/plan';
import Svg, { SvgXml } from 'react-native-svg';

const CastlePlanView = (props: { mapImage: PlanImage, roomList: Room[], selectedRoom?: Room, fullScreenMode?: boolean }) => {

    const DEFAULT_SCALE = 1
    const MIN_SCALE = 0.95

    const MAP_VIEW_SIZE = {
        height: 500,
        width: "100%"
    }

    const { mapImage, roomList, selectedRoom, fullScreenMode } = props;

    const panRef = useRef<React.ReactElement>();
    const pinchRef = useRef<React.ReactElement>();
    const svgRef = useRef<React.ReactElement>();

    const translateX = useSharedValue(0);
    const translateY = useSharedValue(0);
    const scale = useSharedValue(DEFAULT_SCALE);

    console.log(mapImage.svg.substring(0, 500))

    const onPanEvent = useAnimatedGestureHandler({
        // handle one finger -> drag / move
        onStart: (_, ctx: any) => {
            ctx.startX = translateX.value;
            ctx.startY = translateY.value;
        },
        onActive: (event, ctx: any) => {
            translateX.value = ctx.startX + event.translationX;
            translateY.value = ctx.startY + event.translationY;
        },
    });

    const onGestureEvent = useAnimatedGestureHandler({
        onActive: (event: any) => {
            // handle two fingers -> zoom  + - 
            if (event.numberOfPointers === 2) {
                if (event.scale < MIN_SCALE) {
                    scale.value = MIN_SCALE
                }
                else {
                    scale.value = event.scale;
                }
            }
        },
    });

    const animatedStyle = useAnimatedStyle(() => {
        return {
            transform: [
                { translateX: translateX.value },
                { translateY: translateY.value },
                { scale: scale.value },
            ],
        };
    });

    console.log(fullScreenMode)

    console.log(Dimensions.get('window').height)
    console.log(Dimensions.get('window').width)

    return (
        // container
        <Box
            width={!fullScreenMode ? MAP_VIEW_SIZE.width : Dimensions.get('window').width - 5}
            height={!fullScreenMode ? MAP_VIEW_SIZE.height : Dimensions.get('window').height - 62.5}
            // @ts-ignore
            style={
                fullScreenMode ? {
                    marginTop: 2.5,
                    marginLeft: 2.5,
                    marginRight: 2.5,
                }
                    :
                    { borderColor: "#F5F5F", borderWidth: 1, overflow: "hidden" }
            }>
            <PinchGestureHandler
                ref={pinchRef}
                // @ts-ignore
                onGestureEvent={onGestureEvent}
                simultaneousHandlers={[svgRef, panRef]}>
                <Animated.View style={{ flex: 1 }}>
                    <PanGestureHandler
                        ref={panRef}
                        simultaneousHandlers={[svgRef, pinchRef]}
                        onGestureEvent={onPanEvent}
                    >
                        <Animated.View style={[{
                            flex: 1,
                        }, animatedStyle]}>
                            <Svg
                                ref={(ref: any) => (svgRef.current = ref)}
                            >
                                {mapImage && mapImage.viewBox &&
                                    // background image                    
                                    <SvgXml
                                        xml={mapImage.svg}
                                        width={"100%"}
                                        height={"100%"}

                                    />}
                                {mapImage && roomList && roomList.length > 0 &&
                                    <></>}

                            </Svg>
                        </Animated.View>
                    </PanGestureHandler>
                </Animated.View>
            </PinchGestureHandler>
        </Box>
    );
};

export default CastlePlanView;

